﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using EHHangulLibrary;
using System.Windows.Automation;
using System.Collections.Generic;
using System.Diagnostics;

namespace DemoKeyEvent
{
    public partial class Form1 : Form
    {
        bool is_press_shift;
        bool check;
        AutomationFocusChangedEventHandler afceh;
        AutomationElement target_ae;
        ValuePattern target;        
        Hangul hangul;
        List<Button> buttons = new List<Button>();
        public Form1()
        {
            InitializeComponent();
        }

        private void Button_Click(object sender, EventArgs e)
        {            
            Button btn = sender as Button;
            if (target != null)
            {
                target_ae.SetFocus();
                Thread.Sleep(10);
            }
            switch (btn.Text)
            {
                case "Esc": WrapNative.KeyClick(Keys.Escape, is_press_shift); break;
                case "F1": WrapNative.KeyClick(Keys.F1, is_press_shift); break;
                case "F2": WrapNative.KeyClick(Keys.F2, is_press_shift); break;
                case "F3": WrapNative.KeyClick(Keys.F3, is_press_shift); break;
                case "F4": WrapNative.KeyClick(Keys.F4, is_press_shift); break;
                case "F5": WrapNative.KeyClick(Keys.F5, is_press_shift); break;
                case "F6": WrapNative.KeyClick(Keys.F6, is_press_shift); break;
                case "F7": WrapNative.KeyClick(Keys.F7, is_press_shift); break;
                case "F8": WrapNative.KeyClick(Keys.F8, is_press_shift); break;
                case "F9": WrapNative.KeyClick(Keys.F9, is_press_shift); break;
                case "F10": WrapNative.KeyClick(Keys.F10, is_press_shift); break;
                case "F11": WrapNative.KeyClick(Keys.F11, is_press_shift); break;
                case "F12": WrapNative.KeyClick(Keys.F12, is_press_shift); break;                
                case "Tab": WrapNative.KeyClick(Keys.Tab, is_press_shift); break;                
                case "Shift": WrapNative.KeyClick(Keys.ShiftKey, is_press_shift); break;
                case "Enter": WrapNative.KeyClick(Keys.Enter, is_press_shift); break;
                case "Caps": WrapNative.KeyClick(Keys.CapsLock, is_press_shift);
                    cb_cabs.Checked ^= true;
                    ButtonSet();
                    break;
                case "Ctrl": WrapNative.KeyClick(Keys.ControlKey, is_press_shift); break;
                case "Alt": WrapNative.KeyClick(Keys.Alt, is_press_shift); break;                
                case "한/영 변환":  check ^= true;    ButtonSet();        break;                
                case "Space": PressKey(' '); break;
                case "<-": PressKey('\b'); break;
                default: PressKey(btn.Text[0]); break;
            }
            
            lbox_event.Items.Add(btn.Text);
            lbox_event.SelectedIndex = lbox_event.Items.Count - 1;
            WrapNative.KeyClick(Keys.End,is_press_shift);
        }

        private void PressKey(char ch)
        {
            if (target == null)
            {
                return;
            }
            if (check)
            {
                hangul.Input(ch);
            }
            else
            {
                hangul.InputNoKorea(ch);
            }
            lb_text.Text = hangul.Text;
            target.SetValue(hangul.Text);            
        }        
        

        private void cb_cabs_CheckedChanged(object sender, EventArgs e)
        {
            if (cb_cabs.Checked)
            {
                cb_cabs.BackColor = Color.YellowGreen;
            }
            else
            {
                cb_cabs.BackColor = SystemColors.Control;
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {                        
            afceh = new AutomationFocusChangedEventHandler(FocusChanged);
            Automation.AddAutomationFocusChangedEventHandler(afceh);
            ButtonSet();
            
        }
        private void FocusChanged(object sender, AutomationFocusChangedEventArgs e)
        {
            AutomationElement ae = AutomationElement.FocusedElement;
            if (ae == target_ae)
            {
                return;
            }
            if (ae.Current.ProcessId == Process.GetCurrentProcess().Id)
            {
                return;
            }
            if (ae.Current.IsEnabled)
            {
                object obj;
                if (ae.TryGetCurrentPattern(ValuePattern.Pattern, out obj))
                {
                    SetTarget(ae, obj);
                }
                else
                {
                    target_ae = null;
                    target = null;
                }
            }
            else
            {
                target_ae = null;
                target = null;
            }
        }

        delegate void SetTargetDele(AutomationElement ae,object obj);
        private void SetTarget(AutomationElement ae,object obj)
        {
            if(this.InvokeRequired)
            {
                this.Invoke(new SetTargetDele(SetTarget),new object[]{ae,obj});
            }
            else
            {
                target = obj as ValuePattern;
                target_ae = ae;
                hangul = new Hangul();
                hangul.Text = target.Current.Value;
                lb_text.Text = hangul.Text;
                cb_cabs.Checked = WrapNative.IsPress(Keys.CapsLock);
                ButtonSet();
                lbox_event.Items.Add(ae.Current.Name);
            }
        }
        
        private void ButtonSet()
        {   
            if (is_press_shift)
            {
                btn_fpu.Text = "~";
                btn_1.Text = "!";
                btn_2.Text = "@";
                btn_3.Text = "#";
                btn_4.Text = "$";
                btn_5.Text = "%";
                btn_6.Text = "^";
                btn_7.Text = "&&";
                btn_8.Text = "*";
                btn_9.Text = "(";
                btn_0.Text = ")";
                btn_dash.Text = "_";
                btn_equal.Text = "+";
                btn_bs.Text = "|";

                btn_preb.Text = "{";
                btn_postb.Text = "}";

                btn_semi.Text = ":";
                btn_upper.Text = "\"";

                btn_comma.Text = "<";
                btn_dot.Text = ">";
                btn_slush.Text = "?";
            }
            else
            {
                btn_fpu.Text = "`";
                btn_1.Text = "1";
                btn_2.Text = "2";
                btn_3.Text = "3";
                btn_4.Text = "4";
                btn_5.Text = "5";
                btn_6.Text = "6";
                btn_7.Text = "7";
                btn_8.Text = "8";
                btn_9.Text = "9";
                btn_0.Text = "0";
                btn_dash.Text = "-";
                btn_equal.Text = "=";
                btn_bs.Text = "\\";

                btn_preb.Text = "[";
                btn_postb.Text = "]";

                btn_semi.Text = ";";
                btn_upper.Text = "'";

                btn_comma.Text = ",";
                btn_dot.Text = ".";
                btn_slush.Text = "/";
            }
            
            if (check)
            {
                if (is_press_shift)
                {
                    btn_q.Text = "ㅃ";
                    btn_w.Text = "ㅉ";
                    btn_e.Text = "ㄸ";
                    btn_r.Text = "ㄲ";
                    btn_t.Text = "ㅆ";
                    btn_o.Text = "ㅒ";
                    btn_p.Text = "ㅖ";
                }
                else
                {
                    btn_q.Text = "ㅂ";
                    btn_w.Text = "ㅈ";
                    btn_e.Text = "ㄷ";
                    btn_r.Text = "ㄱ";
                    btn_t.Text = "ㅅ";
                    btn_o.Text = "ㅐ";
                    btn_p.Text = "ㅔ";

                }
                btn_y.Text = "ㅛ";
                btn_u.Text = "ㅕ";
                btn_i.Text = "ㅑ";

                btn_a.Text = "ㅁ";
                btn_s.Text = "ㄴ";
                btn_d.Text = "ㅇ";
                btn_f.Text = "ㄹ";
                btn_g.Text = "ㅎ";
                btn_h.Text = "ㅗ";
                btn_j.Text = "ㅓ";
                btn_k.Text = "ㅏ";
                btn_l.Text = "ㅣ";

                btn_z.Text = "ㅋ";
                btn_x.Text = "ㅌ";
                btn_c.Text = "ㅊ";
                btn_v.Text = "ㅍ";
                btn_b.Text = "ㅠ";
                btn_n.Text = "ㅜ";
                btn_m.Text = "ㅡ";
            }
            else
            {
                if (is_press_shift ^ cb_cabs.Checked)
                {
                    btn_q.Text = "Q";
                    btn_w.Text = "W";
                    btn_e.Text = "E";
                    btn_r.Text = "R";
                    btn_t.Text = "T";
                    btn_y.Text = "Y";
                    btn_u.Text = "U";
                    btn_i.Text = "I";
                    btn_o.Text = "O";
                    btn_p.Text = "P";

                    btn_a.Text = "A";
                    btn_s.Text = "S";
                    btn_d.Text = "D";
                    btn_f.Text = "F";
                    btn_g.Text = "G";
                    btn_h.Text = "H";
                    btn_j.Text = "J";
                    btn_k.Text = "K";
                    btn_l.Text = "L";



                    btn_z.Text = "Z";
                    btn_x.Text = "X";
                    btn_c.Text = "C";
                    btn_v.Text = "V";
                    btn_b.Text = "B";
                    btn_n.Text = "N";
                    btn_m.Text = "M";

                }
                else
                {
                    btn_q.Text = "q";
                    btn_w.Text = "w";
                    btn_e.Text = "e";
                    btn_r.Text = "r";
                    btn_t.Text = "t";
                    btn_y.Text = "y";
                    btn_u.Text = "u";
                    btn_i.Text = "i";
                    btn_o.Text = "o";
                    btn_p.Text = "p";


                    btn_a.Text = "a";
                    btn_s.Text = "s";
                    btn_d.Text = "d";
                    btn_f.Text = "f";
                    btn_g.Text = "g";
                    btn_h.Text = "h";
                    btn_j.Text = "j";
                    btn_k.Text = "k";
                    btn_l.Text = "l";


                    btn_z.Text = "z";
                    btn_x.Text = "x";
                    btn_c.Text = "c";
                    btn_v.Text = "v";
                    btn_b.Text = "b";
                    btn_n.Text = "n";
                    btn_m.Text = "m";
                }
            }            
        }

        private void cb_with_shift_CheckedChanged(object sender, EventArgs e)
        {
            is_press_shift = cb_with_shift.Checked;
            ButtonSet();     
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            Automation.RemoveAutomationFocusChangedEventHandler(afceh);
        }       
    }
}
